// Matrix.h: interface for the CMatrix class.
//
//////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2008  Colin Caprani - www.colincaprani.com

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#if !defined(AFX_MATRIX_H__E6397425_F122_44A4_A3A8_53CD4D2523E7__INCLUDED_)
#define AFX_MATRIX_H__E6397425_F122_44A4_A3A8_53CD4D2523E7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <vector>
using namespace std;

template <class T>
class CMatrix  
{
public:
	int dim(int a);
	CMatrix(int a, int b, int c);
	CMatrix();
	virtual ~CMatrix();
	bool checkBounds(int a, int b, int c);
	void add(int a, int b, int c);

	T operator()(const int, const int, const int) const;
	// Get the value of one element.
	T& operator()(const int, const int, const int);
	// Get/Set the value of one element.

private:
	int d3;
	int d2;
	int d1;
	typedef std::vector< std::vector<T> > M2D;

	std::vector< M2D > m;
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

template<class T>  
CMatrix<T>::CMatrix(int a, int b, int c)
{
	d1 = a;	d2 = b; d3 = c;
//	m.assign(d1);

	vector<T> v3;
	v3.assign(d3,0);
	M2D v2;	v2.assign(d2,v3);
	m.assign(d1, v2);

	
/*	for(int i = 0; i < d1; i++)
	{
		m[i].assign(d2,0);
		for(int j = 0; j < d2; j++)
			m[i][j].assign(d3, 0);
	}
*/
}

template<class T>  
CMatrix<T>::CMatrix()
{

}

template<class T>  
CMatrix<T>::~CMatrix()
{

}

template<class T>  
T 
CMatrix<T>::operator()(const int x, const int y, const int z) const
{
	if( checkBounds(x, y, z) )
		return m.at(x-1).at(y-1).at(z-1);
}


template<class T>  
T& 
CMatrix<T>::operator()(const int x, const int y, const int z)
{
	if(checkBounds(x, y, z))
		return m.at(x-1).at(y-1).at(z-1);
	else
	{
		add(x, y, z);
		return m.at(x-1).at(y-1).at(z-1);
	}
}

template<class T>  
int 
CMatrix<T>::dim(int a)
{
	if(a == 1)
		return d1;
	else if(a == 2)
		return d2;
	else
		return d3;
}

template<class T>  
void
CMatrix<T>::add(int a, int b, int c)
{
	vector<T> v3;
	v3.assign(d3,0);	
	M2D v2;	v2.assign(d2,v3);
	m.assign(d1, v2);
	
	if(a > d1)
	{
		d1 = a;
		m.resize(d1,v2);
	}
	if(b > d2)
	{
		d2 = b;
		for(int i = 0; i < d1; i++)
			m[i].resize(d2, v3);
	}
	if(c > d3)
	{
		d3 = c;
		for(int i = 0; i < d1; i++)
		{
			for(int j = 0; j < d2; j++)
				m[i][j].resize(d3, 0);
		}
	}
}

template<class T>  
inline bool 
CMatrix<T>::checkBounds(int a, int b, int c)
{
	if(a-1 < d1 || b-1 < d2 || c-1 < d3)
		return 1;
	else
		return 0;
}


#endif // !defined(AFX_MATRIX_H__E6397425_F122_44A4_A3A8_53CD4D2523E7__INCLUDED_)
